package org.greenstone.gatherer.util;

import java.io.*;
import java.util.*;

public class StatusHTML {

    static final private String DARK_GREEN  = "B0D0B0";
    static final private String DARK_RED    = "D0B0B0";
    static final private String LIGHT_GREEN = "E0F0E0";
    static final private String LIGHT_RED   = "F0E0E0";

    static final private String DARK_BLUE   = "B0B0D0";
    static final private String LIGHT_BLUE  = "E0E0F0";

    static final private String FIXED_STR   = "Fixed";
    static final private String NOTHING_STR   = "Nothing";

    static final private String FEATURES = "  <!-- Header -->\n  <TR bgcolor=\"#B0B0D0\">\n    <TD colspan=\"2\"><STRONG><A name=\"23b\">GLI v2.3 beta - Feature Requests [FEATURE_COUNT total]</A></STRONG></TD>\n  </TR>\n  <TR>\n    <TD bgcolor=\"#FFFFFF\" colspan=\"2\">In order received. Similar problems only mentioned once, with descriptions combined where possible.</TD>\n  </TR>\n  <TR>\n    <TD colspan=\"2\">&#160;</TD>\n  </TR>\n";

    static final private String FIXED = "  <!-- Header -->\n  <TR bgcolor=\"#B0D0B0\">\n    <TD colspan=\"2\"><STRONG><A name=\"23b\">GLI v2.3 beta - Closed Reports [FIXED_COUNT total]</A></STRONG></TD>\n  </TR>\n  <TR>\n    <TD bgcolor=\"#FFFFFF\" colspan=\"2\">In order received. Similar problems only mentioned once, with descriptions combined where possible.</TD>\n  </TR>\n  <TR>\n    <TD colspan=\"2\">&#160;</TD>\n  </TR>\n";

    static final private String FOOTER = "  <TR><TD>&nbsp;</TD></TR>\n</TABLE>\n<TABLE bgcolor=\"#FFFFFF\" width=\"100%\" border=\"0\" cellpadding=\"5\" cellspacing=\"0\">\n  <TR>\n    <TD align=\"center\">Written by John Thompson. Any comments, bug reports email <A href=\"mailto:jmt12@cs.waikato.ac.nz\">here</A>.</TD>\n  </TR>\n</TABLE>\n</BODY>\n</HTML>";
    static final private String HEADER = "<HTML>\n<HEAD>\n<TITLE>GLI Project Status</TITLE>\n</HEAD>\n<BODY bgcolor=\"#CCCCCC\">\n<TABLE bgcolor=\"#FFFFFF\" width=\"100%\" border=\"0\" cellpadding=\"5\" cellspacing=\"0\">\n  <!-- Header -->\n  <TR bgcolor=\"#B0D0B0\">\n    <TD align=\"center\" colspan=\"7\"><FONT size=\"4\"><STRONG>Greenstone Librarian Interface (GLI) version 2.3 beta</STRONG></FONT></TD>\n  </TR>\n  <TR>\n    <TD colspan=\"7\">This page lists the features planned for the Greenstone Librarian Interface in its next release. It also details the various bugs that have been found and reported in this version of the GLI. You can also review the status reports for these other versions:</TD>\n  </TR>\n  <TR bgcolor=\"#B0D0B0\">\n    <TD align=\"center\">Status v2.3 beta</TD>\n    <TD align=\"center\"><A href=\"http://www.greenstone.org/gatherer/status_23c.html\">Status v2.3c</A></TD>\n    <TD align=\"center\"><A href=\"http://www.greenstone.org/gatherer/status_23b.html\">Status v2.3b</A></TD>\n    <TD align=\"center\"><A href=\"http://www.greenstone.org/gatherer/status_23a.html\">Status v2.3a</A></TD>\n    <TD align=\"center\"><A href=\"http://www.greenstone.org/gatherer/status_22b.html\">Status v2.2b</A></TD>\n    <TD align=\"center\"><A href=\"http://www.greenstone.org/gatherer/status_21.html\">Status v2.1</A></TD>\n    <TD align=\"center\"><A href=\"http://www.greenstone.org/gatherer/status_20.html\">Status v2.0</A></TD>\n  </TR>\n</TABLE>\n<BR/>\n<TABLE border=\"0\" cellpadding=\"5\" cellspacing=\"0\" width=\"100%\">\n";
    
    static final private String PENDING = "  <TR><TD bgcolor=\"#CCCCCC\">&nbsp;</TD></TR>\n  <!-- Header -->\n  <TR bgcolor=\"#D0B0B0\">\n    <TD colspan=\"2\"><STRONG><A name=\"23b\">GLI v2.3 beta - Current Reports [PENDING_COUNT total]</A></STRONG></TD>\n  </TR>\n  <TR>\n    <TD bgcolor=\"#FFFFFF\" colspan=\"2\">In order received. Similar problems only mentioned once, with descriptions combined where possible.</TD>\n  </TR>\n  <TR>\n    <TD colspan=\"2\">&#160;</TD>\n  </TR>\n";

    static final private String[] PREFIX = { 	"  <TR><TD bgcolor=\"#DARK_COLOR\" valign=\"top\">ID#</TD>              <TD bgcolor=\"#DARK_COLOR\">","  <TR><TD bgcolor=\"#DARK_COLOR\" valign=\"top\">Description</TD>      <TD bgcolor=\"#LIGHT_COLOR\">","  <TR><TD bgcolor=\"#DARK_COLOR\" valign=\"top\">Reported By</TD>      <TD bgcolor=\"#FFFFFF\">","  <TR><TD bgcolor=\"#DARK_COLOR\" valign=\"top\">Platform</TD>         <TD bgcolor=\"#FFFFFF\">","  <TR><TD bgcolor=\"#DARK_COLOR\" valign=\"top\">Test Item#</TD>       <TD bgcolor=\"#FFFFFF\">","  <TR><TD bgcolor=\"#DARK_COLOR\" valign=\"top\">Status</TD>           <TD bgcolor=\"#FFFFFF\">","  <TR><TD bgcolor=\"#DARK_COLOR\" valign=\"top\">Proposed Solution</TD><TD bgcolor=\"#FFFFFF\">","  <TR><TD bgcolor=\"#DARK_COLOR\" valign=\"top\">Solution</TD><TD bgcolor=\"#FFFFFF\">",  "  <TR><TD bgcolor=\"#CCCCCC\">&nbsp;"};
    
    static final private String SUFFIX = "</TD></TR>\n";
    
    static public void main(String[] args) {
	File in_file = null;
	File out_file = null;
	if(args.length == 2) {
	    in_file = new File(args[0]);
	    if(!in_file.exists()) {
		System.out.println("Can't find source file: " + in_file.getAbsolutePath());
		in_file = null;
	    }
	    out_file = new File(args[1]);
	}
	if(in_file != null && out_file != null) {
	    new StatusHTML(in_file, out_file);
	}
	else {
	    System.out.println("Usage: java org.greenstone.gatherer.util.StatusHTML <input_file> <output_file>");
	}
    }

    public StatusHTML(File in_file, File out_file) {
	LinkedList reports = new LinkedList();
	try {
	    int feature_count = 0;
	    int fixed_count = 0;
	    int pending_count = 0;
	    BufferedReader in = new BufferedReader(new FileReader(in_file));
	    // Read in the entire file, building report objects.
	    String line = null;
	    while((line = in.readLine()) != null) {
		// We may of course end up with several nulls, but thats hardly our fault.
		Report report = new Report(line, in.readLine(), in.readLine(), in.readLine(), in.readLine(), in.readLine(), in.readLine());
		if(report.isFixed()) {
		    fixed_count++;
		}
		else if (report.isFeatureRequest()) {
		    feature_count++;
		}
		else {
		    pending_count++;
		}
		reports.add(report);
		report = null;
		// Throw away a line
		in.readLine();
	    }
	    in.close();
	    in = null;

	    // Create and start writing the output file
	    FileOutputStream out = new FileOutputStream(out_file);
	    out.write(HEADER.getBytes());

	    // Write out all the pending jobs
	    out.write((PENDING.replaceAll("PENDING_COUNT", String.valueOf(pending_count))).getBytes());

	    // Write out the bugs
	    for (int i = 0; i < reports.size(); i++) {
		Report report = (Report) reports.get(i);
		if (!report.isFixed() && !report.isFeatureRequest()) {
		    out.write(report.getBytes(DARK_RED, LIGHT_RED));
		}
	    }

	    // Write out all the feature requests
	    out.write((FEATURES.replaceAll("FEATURE_COUNT", String.valueOf(feature_count))).getBytes());

	    // Write out the feature requests
	    for (int i = 0; i < reports.size(); i++) {
		Report report = (Report) reports.get(i);
		if (!report.isFixed() && report.isFeatureRequest()) {
		    out.write(report.getBytes(DARK_BLUE, LIGHT_BLUE));
		}
		report = null;
	    }

	    // Write out everything else
	    out.write((FIXED.replaceAll("FIXED_COUNT", String.valueOf(fixed_count))).getBytes());
	    for(int i = 0; i < reports.size(); i++) {
		Report report = (Report) reports.get(i);
		if(report.isFixed()) {
		    out.write(report.getBytes(DARK_GREEN, LIGHT_GREEN));
		}
		report = null;
	    }

	    // Write the footer and close the file
	    out.write(FOOTER.getBytes());
	    out.close();
	    out = null;
	}
	catch(Exception error) {
	    error.printStackTrace();
	}
    }

    private class Report {
	public String bug_id;
	public String description;
	public String platform;
	public String reported_by;
	public String status;
	public String solution;
	public String test_item;

	public Report(String bug_id, String description, String reported_by, String platform, String test_item, String status, String solution) {
	    if(bug_id != null) this.bug_id = bug_id.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
	    if(description != null) this.description = description.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
	    if(platform != null) this.platform = platform.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
	    if(reported_by != null) this.reported_by = reported_by.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
	    if(solution != null) this.solution = solution.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
	    if(status != null) this.status = status;  // status.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
	    if(test_item != null) this.test_item = test_item;  // test_item.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
	}

	public boolean isFixed() {
	    return (status != null && (status.startsWith(FIXED_STR) || status.startsWith(NOTHING_STR)));
	}

	public boolean isFeatureRequest() {
	    return (bug_id.indexOf("F") != -1);
	}

	public byte[] getBytes(String dark_color, String light_color) {
	    StringBuffer text_buffer = new StringBuffer();
	    text_buffer.append(PREFIX[0]);
	    text_buffer.append(bug_id);
	    text_buffer.append(SUFFIX);
	    text_buffer.append(PREFIX[1]);
	    text_buffer.append(description);
	    text_buffer.append(SUFFIX);
	    text_buffer.append(PREFIX[2]);
	    text_buffer.append(reported_by);
	    text_buffer.append(SUFFIX);
	    text_buffer.append(PREFIX[3]);
	    text_buffer.append(platform);
	    text_buffer.append(SUFFIX);
	    text_buffer.append(PREFIX[4]);
	    text_buffer.append(test_item);
	    text_buffer.append(SUFFIX);
	    text_buffer.append(PREFIX[5]);
	    text_buffer.append(status);
	    text_buffer.append(SUFFIX);
	    if(isFixed()) {
		text_buffer.append(PREFIX[6]);
	    }
	    else {
		text_buffer.append(PREFIX[7]);
	    }
	    text_buffer.append(solution);
	    text_buffer.append(SUFFIX);
	    text_buffer.append(PREFIX[8]);
	    text_buffer.append(SUFFIX);
	    text_buffer.append("\n");
	    String text = text_buffer.toString();
	    text = text.replaceAll("DARK_COLOR", dark_color);
	    text = text.replaceAll("LIGHT_COLOR", light_color);
	    return text.getBytes();
	}
    }
}
